/*
****************************************************************************
****************************************************************************
**                                                                        **
**                XXXX     X    X         X XXXXXX X                      **
**               X    X   X X   X    X    X X      X   X                  **
**               X       X   X  X   X X   X X      X  X                   **
**               X   XX XXXXXXX X  X   X  X XXXX   X X                    **
**               X    X X     X X X     X X X      XX                     **
**               X    X X     X XX       XX X      X                      **
**                XXXX  X     X X         X XXXXXX XXXXXX                 **
**                                                                        **
**                                                                        **
**                           BIALYSTOK 2018                               **
**                                                                        **
**                          MODULE WEBServer                              **
**                                                                        **
**                     Mikrokontroler ARM: LPC2378                        **
**                                                                        **
**                     Wersja z dnia : 2018.06.12                         **
**                     Napisal : Andrzej Pawluczuk                        **
**                                                                        **
****************************************************************************
****************************************************************************
*/

#include <LPC23xx.H>
#include "system.h"
#include "types.h"
#include "systime.h"
#include "irqvectors.h"
#include "serialconst.h"
#include "uart0.h"
#include "strings.h"
#include "emacdriver.h"
#include "ipservice.h"
#include "icmpservice.h"
#include "frameutil.h"
#include "trace.h"

#define IRQCallCntLimit                 100

#define BufferSize               64

typedef struct {
                 EMACInstanceRecT *          EMACLayer ;
                 IPLayerInstanceRecT *       IPLayer ;
                 UCHAR                       Buffer [ BufferSize + 1 ] ;
                 USHORT                      Index ;
                 PtrTimeQueueRecT            NotifyHandle ;
               } ICMPExampeInstanceRecT ;
typedef ICMPExampeInstanceRecT * PtrICMPExampeInstanceRecT ;

static ICMPExampeInstanceRecT ICMPExampeInstance ;
static ULONG IRQCallCounter ;

#define MachineIPAddress        0xC0A80037
//                              ^^^ 192.168.0.55
#define MachineIPSubnet         0xFFFFFF00
//                              ^^^ 255.255.255.0
#define MachineGWIPAddress      0xC0A800FE
//                              ^^^ 192.168.0.254

UCHAR MachineMACAddr [ 6 ]     = { 0x08 , 0x14 , 0x16 , 0x22 , 0x28 , 0x2D } ;


static __irq void T1_IRQHandler ( void )
{
  /*-------------------------------------------------------------------------*/
  IRQCallCounter ++ ;
  if ( IRQCallCounter >= IRQCallCntLimit )
  {
    IRQCallCounter = 0 ;
  } /* if */ ;
  T1IR = 1 ;
  VICVectAddr = 0 ;
} /* T1_IRQHandler */


static void HardwareInit ( void ) 
{
  /*-------------------------------------------------------------------------*/
  PINSEL10 = 0 ;
  T1MR0 = ( PCLK / 1000 ) - 1 ;
  T1MCR = 3 ;
  T1TCR = 1 ;
  VICVectAddr5 = ( unsigned long ) T1_IRQHandler ;
  VICVectCntl5 = 15 ;
  VICIntEnable = ( 1 << TIMER1_Interrupt ) ;
  UART0InitEnvir ( ) ;
} /* HardwareInit */


static void StartNet ( PtrICMPExampeInstanceRecT Instance )
{
  EtherSpeedMode EthMode ;
  UCHAR LocalMyMACAddr [ 6 ] ;
  USHORT Loop ;
  /*-------------------------------------------------------------------------*/
  for ( Loop = 0 ; Loop < 6 ; Loop ++ )
    LocalMyMACAddr [ Loop ] = MachineMACAddr [ Loop ] ;
  EthMode = Mode_AutoNeg ;
//  EthMode = Mode_10MBIT ;
//  EthMode = Mode_100MBIT ;
  Instance -> EMACLayer = InitPHYLayerInstance ( LocalMyMACAddr , EthMode ) ;
  Instance -> IPLayer = InitIPLayerInstance ( Instance -> EMACLayer ,
                                              MachineIPAddress , MachineIPSubnet ) ;
  Instance -> IPLayer -> MyDefaultGWIPAddress = MachineGWIPAddress ;
} /* StartNet */


UCHAR Hello1 [ ]      = "\r\nHello: obsluga ICMP\r\n" ;
UCHAR Hello2 [ ]      = "\r\nMoj adres IP=" ;
UCHAR Hello3 [ ]      = "\r\nMaska podsieci=" ;
UCHAR Hello4 [ ]      = "\r\nAdres bramy domyslnej=" ;
UCHAR Hello6 [ ]      = "\r\nAdres MAC=" ;
UCHAR Hello7 [ ]      = "\r\n************************\r\n" ;

static void HelloMessage ( void )
{
  /*-------------------------------------------------------------------------*/
  UART0SendString ( Hello1 ) ;
  UART0SendString ( Hello2 ) ;
  TraceIP ( MachineIPAddress ) ;
  UART0SendString ( Hello3 ) ;
  TraceIP ( MachineIPSubnet ) ;
  UART0SendString ( Hello4 ) ;
  TraceIP ( MachineGWIPAddress ) ;
  UART0SendString ( Hello6 ) ;
  TraceMAC ( MachineMACAddr ) ;
  UART0SendString ( Hello7 ) ;
} /* HelloMessage */


static void SoftwareInit ( void )
{
  /*-------------------------------------------------------------------------*/
  IRQCallCounter = 0 ;
  ICMPExampeInstance . EMACLayer = NIL ;
  ICMPExampeInstance . IPLayer = NIL ;
  ICMPExampeInstance . Index = 0 ;
  ICMPExampeInstance . Buffer [ 0 ] = 0 ;
  ICMPExampeInstance . NotifyHandle = NIL ;
} /* SoftwareInit */


UCHAR Message10 [ ]        = "\r\nW rozsadnym czasie nie przyszla odpowiedz na PING\r\n" ;

static void EchoReplyTimeOut (  void * CallParam )
{
  /*-------------------------------------------------------------------------*/
  UART0SendString ( Message10 ) ;
} /* EchoReplyTimeOut */

#define ICMP_DataOffset               EtherPackedHeaderRecTSize + IPHeaderSize + 4

const UCHAR PingText [ ]     = "*** Kocham swiat ***" ;
static UCHAR Message01 [ ]   = "\r\nPrzetwarzanie odpowiedzi na prosbe o echo\r\nTYPE=" ;
static UCHAR Message02 [ ]   = " CODE=" ;
static UCHAR Message03 [ ]   = " HCRC=" ;
static UCHAR Message04 [ ]   = "\r\nIDENT=" ;
static UCHAR Message05 [ ]   = " SEQ=" ;
static UCHAR Message06 [ ]   = "\r\nDATA=" ;
static UCHAR Message07 [ ]   = "Echo jest poprawne :-)\r\n" ;
static UCHAR Message08 [ ]   = "Echo jest przeklamane :-(\r\n" ;

static void AcceptEchoReply ( PtrIPLayerInstanceRecT IPLayerInstance ,
                              IPHeaderRecT *         IPHeader ,
                              PtrOSFrameRecT         Frame )
{
  USHORT TypeAndCode ;
  USHORT HCRC ;
  USHORT Ident ;
  USHORT Seq ;
  USHORT ICMPDataSize ;
  UCHAR * Ch ;
  USHORT Loop ;
  UCHAR ValidReply ;
  const UCHAR * PCh ;
  /*-------------------------------------------------------------------------*/
  ValidReply = TRUE ;
  ICMPDataSize = IPHeader -> TotalLength - IPHeaderSize - 4 ;
  TypeAndCode = GetUSHORTSwapField ( Frame , IPDataOffset ) ;
  HCRC = GetUSHORTSwapField ( Frame , IPDataOffset + 2 ) ;
  Ident = GetUSHORTSwapField ( Frame , IPDataOffset + 4 ) ;
  Seq = GetUSHORTSwapField ( Frame , IPDataOffset + 6 ) ;
  UART0SendString ( Message01 ) ;
  TraceUCHAR ( ( UCHAR ) ( TypeAndCode >> 8 ) ) ;
  UART0SendString ( Message02 ) ;
  TraceUCHAR ( ( UCHAR ) ( TypeAndCode & 0xFF ) ) ;
  UART0SendString ( Message03 ) ;
  TraceUSHORT ( HCRC ) ;
  UART0SendString ( Message04 ) ;
  TraceUSHORT ( Ident ) ;
  UART0SendString ( Message05 ) ;
  TraceUSHORT ( Seq ) ;
  UART0SendString ( Message06 ) ;
  if ( Ident != 12 )
    ValidReply = FALSE ;
  if ( Seq != 34 )
    ValidReply = FALSE ;
  Ch = & Frame -> Data [ ICMP_DataOffset + 4 ] ;
  PCh = PingText ;
  for ( Loop = 4 ; Loop < ICMPDataSize ; Loop ++ )
  {
    if ( ( * PCh ) != ( * Ch ) )
      ValidReply = FALSE ;
    if ( * Ch < 32 )
    {
      UART0Send ( '<' ) ;
      TraceUCHAR ( * Ch ) ;
      UART0Send ( '>' ) ;
    } /* if ... */
    else
    {
      UART0Send ( * Ch ) ;
    } /* if ... else */ ;
    Ch ++ ;
    PCh ++ ;
  } /* for */ ;
  UART0Send ( 0x0D ) ;
  UART0Send ( 0x0A ) ;
  if ( ValidReply )
    UART0SendString ( Message07 ) ;
  else
    UART0SendString ( Message08 ) ;
  if ( ICMPExampeInstance . NotifyHandle )
    DetachTimer ( ICMPExampeInstance . NotifyHandle ) ;
  ICMPExampeInstance . NotifyHandle = NIL ;
} /* AcceptEchoReply */


UCHAR TraceInput1 [ ]       = "\r\nWczytano: " ;
UCHAR TraceInput2 [ ]       = "\r\n" ;

static void SerialPool ( void )
{
  UCHAR Ch ;
  ULONG DersinationIP ;
  /*-------------------------------------------------------------------------*/
  if ( UART0DataPresent ( ) )
  {
    Ch = UART0GetData ( ) ;
    if ( Ch == 0x0D )
    {
      UART0SendString ( TraceInput1 ) ;
      UART0SendString ( ICMPExampeInstance . Buffer ) ;
      UART0SendString ( TraceInput2 ) ;     
      if ( StrToIP ( ICMPExampeInstance . Buffer , & DersinationIP ) )
      {
        ICMPExampeInstance . NotifyHandle = AttachTimer ( 1000 , EchoReplyTimeOut , NIL ) ;
        SendPingTo ( ICMPExampeInstance . IPLayer , DersinationIP , PingText ) ;
      } /* if */ ;
      ICMPExampeInstance . Index = 0 ;
      ICMPExampeInstance . Buffer [ 0 ] = 0 ;
      return ;
    } /* if */ ;
    if ( ICMPExampeInstance . Index < BufferSize )
    {
      ICMPExampeInstance . Buffer [ ICMPExampeInstance . Index ] = Ch ;
      ICMPExampeInstance . Index ++ ;
      ICMPExampeInstance . Buffer [ ICMPExampeInstance . Index ] = 0 ;
    } /* if */ ;
  } /* if */ ;
} /* SerialPool */


int main ( void )
{
  /*-------------------------------------------------------------------------*/
  SysInit ( ) ;
  InitSysTime ( ) ;
  HardwareInit ( )  ;
  SoftwareInit ( ) ;
  UART0HardwInit( FOSC , N81Mode , 9600 ) ;
  HelloMessage ( ) ;
  StartNet ( & ICMPExampeInstance ) ;
  SetICMPEchoService ( ICMPExampeInstance . IPLayer , AcceptEchoReply ) ;
  for ( ; ; )
  {
    SysTimerPool ( ) ;
    ICMPExampeInstance . IPLayer -> IPLayerPoolService ( ICMPExampeInstance . IPLayer ) ;
    SerialPool ( ) ;
  } /* loop */ ;
} /* main */

